%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TAR_RATE.M
% To estimate and test a threshold bi-variate VECM
% on term structure data

% written by:

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu
% http://www.ssc.wisc.edu/~bhansen/

% and 

% Byeongseon Seo
% Department of Economics
% Soongsil University
% Seoul, 156-743
% Korea
% seo@saint.soongsil.ac.kr


% This Gauss program estimates a bi-variate VECM, a threshold bi-variate VECM, 
% and tests for the presence of a threshold.  The methods are those described in
% "Testing for Threshold Cointegration" by Bruce E. Hansen and Byeongseon Seo.

% This particular program replicates the empirical work reported in that paper.
% For a more generic version of the program, use "tar_ci.prg".



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ur_rate()
global ffff;
ffff=0;
trim_=.05;
boot=5000;
load zeroyld.txt;
dat=zeroyld;
datstore=dat(:,7:62);
rs=[(0:18)';21;24;30;(36:12:120)'];

rates_i=[1 2;
1 3;
1 6;
3 6;
3 12;
3 120;
12 24;
12 120;
24 120];
out=fopen('ur_rate.txt','wt');
fclose(out);
k=1;			% Lags in VAR beyond EC @
while k<=2
    ri=1;
    while ri<=length(rates_i(:,1))
        out=fopen('ur_rate.txt','at');
        short=rates_i(ri,1);
        long=rates_i(ri,2);
        [temp,short_i]=max(rs==short);
        [temp,long_i]=max(rs==long);
        dat=datstore(:,[long_i,short_i]);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        fprintf(out,'*************************************************\n');
        fprintf(out,'Long Rate  (month):                               %u\n',long);
        fprintf(out,'Short Rate (month):                               %u\n',short);
        fprintf(out,'Number of VAR lags:                               %u\n',k);
        fprintf(out,'\n');
        fclose(out);
        % Cointegration Tests %
        % Known CI vector %
        y=dat(:,1)-dat(:,2);
        t=length(y(:,1))-1-k;
        % ADF test %
        dy=y(2+k:t+k+1)-y(1+k:t+k);
        x=[y(1+k:t+k),ones(t,1)];
        j=1;
        while j<=k
            x=[x,y(2+k-j:t+k+1-j)-y(1+k-j:t+k-j)];
            j=j+1;
        end;
        rho=(dy'/x')';
        e=dy-x*rho;
        v=inv(x'*x)*(e'*e)/(t-k-2);
        adf=rho(1)/sqrt(v(1,1));
        out=fopen('ur_rate.txt','at');
        fprintf(out,'ADF test:        %f\n',adf);
        % Estimated CI vector %
        y=dat(:,1);
        x=[ones(length(y(:,1)),1),dat(:,2)];
        beta=(y'/x')';
        y=y-x*beta;
        t=length(y(:,1))-1-k;
        % Engle-Granger test %
        dy=y(2+k:t+k+1)-y(1+k:t+k);
        x=[y(1+k:t+k),ones(t,1)];
        j=1;
        while j<=k
            x=[x,y(2+k-j:t+k+1-j)-y(1+k-j:t+k-j)];
            j=j+1;
        end;
        rho=(dy'/x')';
        e=dy-x*rho;
        v=inv(x'*x)*(e'*e)/(t-k-2);
        adf=rho(1)/sqrt(v(1,1));
        out=fopen('ur_rate.txt','at');
        fprintf(out,'Estimated CI vector: \n');
        fprintf(out,'   %f\n',beta);
        fprintf(out,'Engle-Granger test:     %f\n\n',adf);
        fclose(out);
        
        % Nonlinear Tests %
        y=dat(:,1)-dat(:,2);
        [tests,pvalues]=tar(y,k+1,0,trim_,1-trim_,boot,0,0);
        out=fopen('ur_rate.txt','at');
        fprintf(out,'Test for TAR in EC - Known CI Vector: %f\n',pvalues(1));
        fclose(out);
        ffff=ffff+1;
        y=dat(:,1)-[ones(length(y(:,1)),1),dat(:,2)]*beta;
        [tests,pvalues]=tar(y,k+1,0,trim_,1-trim_,boot,0,0);
        out=fopen('ur_rate.txt','at');
        fprintf(out,'Test for TAR in EC - Estimated CI Vector: %f\n\n\n\n',pvalues(1));
        fclose(out);
        ffff=ffff+1;
        ri=ri+1;
    end;
    k=k+1;
end;
        
        